@@ -17,7 +17,7 @@ $SIG{CHLD} = \&reapChild;
 
 BEGIN{
    use vars qw/$VERSION/;
-   $VERSION = '.07';
+   $VERSION = '.09';
 }
 
 # Options added as needed
@@ -26,7 +26,7 @@ sub new{
 
    my $self = {
       _host          => $arg{host},
-      _user          => $arg{user},
+      _user          => $arg{user} || $ENV{'USER'},
       _password      => $arg{password},
       _cipher        => $arg{cipher},
       _port          => $arg{port},
@@ -35,10 +35,12 @@ sub new{
       _recursive     => $arg{recursive} || 0,
       _verbose       => $arg{verbose} || 0,
       _auto_yes      => $arg{auto_yes} || 0,
+      _terminator    => $arg{terminator} || "\n",
       _timeout       => $arg{timeout} || 10,
       _timeout_auto  => $arg{timeout_auto} || 1,
       _timeout_err   => $arg{timeout_err} || undef,
       _no_check      => $arg{no_check} || 0,
+      _protocol      => $arg{protocol} || undef,
    };
 
    bless($self,$class);
@@ -46,7 +48,7 @@ sub new{
 
 sub _get{
    my($self,$attr) = @_;
-   croak("No attribute supplied to 'get()' method") unless defined $attr;
+
    return $self->{"_$attr"};
 }
 
@@ -115,13 +117,12 @@ sub scp{
    my $recursive    = $self->_get('recursive');
    my $verbose      = $self->_get('verbose');
    my $preserve     = $self->_get('preserve');
-   my $host         = $self->_get('host');
    my $handler      = $self->_get('error_handler');
    my $auto_yes     = $self->_get('auto_yes');
    my $no_check     = $self->_get('no_check');
+   my $terminator   = $self->_get('terminator');
+   my $protocol     = $self->_get('protocol');
  
-   my($file,$reverse);
-
    ##################################################################
    # If the second argument is not provided, the remote file will be
    # given the same (base) name as the local file (or vice-versa).
@@ -130,38 +131,23 @@ sub scp{
       $to = basename($from);
    }  
 
-   # This is probably lame.  Bite me.
-   if($from =~ /(\w+)?:(.*?)$/){
-      my $temp = $to;
-      $to = $from;
-      $from = $temp;
-      $reverse++;
-   }
+   my($host,$dest);
 
-   if($to =~ /^(\w+)@(\w+):(.*?)$/){
-      $login = $1 || $login;
-      $host  = $2 || $host;
-      $file  = $3 || basename($from);
+   # Parse the to/from string. If the $from contains a ':', assume it is the remote host
+   if($from =~ /:/){
+      ($login,$host,$dest) = $self->_parse_scp_string($from);
+      $from = "$login\@$host:";
+      $from .= "$dest" if $dest;
    }
-   elsif($to =~ /^(\w+):(.*?)$/){
-      $host = $1 || $host;
-      $file = $2 || basename($from);
-   }
-   elsif($to =~ /^:?(.*?)$/){
-      $file = $1 || basename($from);
-   }
-   else{}
-
-   $to = "$login\@$host:$file";
-
-   if($reverse){
-      my $temp = $to;
-      $to = $from;
-      $from = $temp;
+   else{
+      ($login,$host,$dest) = $self->_parse_scp_string($to);
+      $to = "$login\@$host:";
+      $to .= "$dest" if $dest;
    }
 
    croak("No login. Can't scp") unless $login;
    croak("No password. Can't scp") unless $password;
+   croak("No host specified. Can't scp") unless $host;
 
    # Gather flags.
    my $flags;
@@ -171,12 +157,15 @@ sub scp{
    $flags .= "-r " if $recursive;
    $flags .= "-v " if $verbose;
    $flags .= "-p " if $preserve;
+   $flags .= "-$protocol " if $protocol;
    $flags .= "-q ";  # Always pass this option (no progress meter)
 
    my $scp = Expect->new;
-   #$scp->raw_pty(1); # Don't take a chance on an echo'd password
+   #if($verbose){ $scp->raw_pty(1) }
+   #$scp->debug(1);
 
-   $scp = Expect->spawn("scp $flags $from $to") or croak "Couldn't start program: $!\n";
+   my $scp_string = "scp $flags $from $to";
+   $scp = Expect->spawn($scp_string) or croak "Couldn't start program: $!\n";
 
    $scp->log_stdout(0);
 
@@ -187,7 +176,7 @@ sub scp{
    }
 
    unless($scp->expect($timeout,-re=>'[Pp]assword:|[Pp]assphrase:')){
-      my $err = $scp->before();
+      my $err = $scp->before() || $scp->match();
       if($err){
          if($handler){ $handler->($err) }
          croak("Problem performing scp: $err");
@@ -197,24 +186,29 @@ sub scp{
 
    if($verbose){ print $scp->before() }
 
-   #$scp->log_file(\&handleErr);
-   $scp->send("$password\n");
+   $password .= $terminator if $terminator;
 
-   #############################################################
+   $scp->send($password);
+
+   ################################################################
    # Check to see if we sent the correct password, or if we got
    # some other bizarre error.  Anything passed back to the
    # terminal at this point means that something went wrong.
-   #############################################################
-   unless($no_check){
+   #
+   # The exception to this is verbose output, which can mistakenly
+   # be picked up by Expect.
+   ################################################################
+   unless($no_check || $verbose){
 
       $scp->expect($timeout_err,
          [qr/[Pp]ass.*/ => sub{
+               print "WTF?\n";
                my $error = $scp->before() || $scp->match();
                if($handler){
                   $handler->($error);
                }
                else{
-                  croak("Error: Bad password");
+                  croak("Error: Bad password [$error]");
                }
             }
          ],
@@ -238,19 +232,35 @@ sub scp{
    if($verbose){ print $scp->after(),"\n" }
 
    $scp->hard_close();
-   return;
+   return 1;
 }
 
-#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-# As far as I can tell, just about *all* output, regardless of whether or not
-# it's a true error, will get sent here.  Hence, the regex check.
-#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-#sub handleErr{
-#   my $err = shift;
-#   if($err =~ /permission denied/i){
-#      croak("Invalid password");
-#   }
-#}
+# Break the from/to line into its various parts
+sub _parse_scp_string{
+   my($self,$string) = @_;
+   my @parts;
+   my($user,$host,$dest);
+
+   @parts = split(/@/,$string);
+   if(scalar(@parts) == 2){
+      $user = shift(@parts);
+   }
+   else{
+      $user = $self->_get("user");
+   }
+
+   my $temp = join('',@parts);
+   ($host,$dest) = split(/:/,$temp);
+
+   # scp('file','file') syntax, where local to remote is assumed
+   unless($dest){
+      $dest = $host;
+      $host = $self->_get("host");
+   }
+
+   $host ||= $self->_get("host");
+   return ($user,$host,$dest);
+}
 
 sub reapChild{
    do {} while waitpid(-1,WNOHANG) > 0;
@@ -272,7 +282,7 @@ C<< $scpe->login('user name', 'password'); >>
 
 C<< $scpe->scp('file','host:/some/dir'); >>
 
-.
+
 
 B<Example 2 - uses constructor, shorthand scp:>
 
@@ -280,7 +290,7 @@ C<< my $scpe = Net::SCP::Expect->new(host=>'host', user=>'user', password=>'xxxx
 
 C<< $scpe->scp('file','/some/dir'); # 'file' copied to 'host' at '/some/dir' >>
 
-.
+
 
 B<Example 3 - Copying from remote machine to local host>
 
@@ -296,7 +306,7 @@ See the B<scp()> method for more information on valid syntax.
 Expect 1.14.  May work with earlier versions, but was tested with 1.14 (and now 1.15)
 only.
 
-Term::ReadPassword 0.01 is required if you want to perform the tests.
+Term::ReadPassword 0.01 is required if you want to execute the interactive test script.
 
 =head1 DESCRIPTION
 
@@ -346,7 +356,6 @@ B<user(>I<user>B<)>
 
 Sets the user for the current object
 
-
 B<scp()>
 
 Copies the file from source to destination.  If no host is specified, you
@@ -396,8 +405,14 @@ B<port> - Use the specified port.
 B<preserve> - Preserves modification times, access times, and modes from
 the original file.
 
+B<protocol> - Specify the ssh protocol to use for scp.  The default is undef,
+which simply means scp will use whatever it normally would use.
+
 B<recursive> - Set to 1 if you want to recursively copy entire directories.
 
+B<terminator> - Set the string terminator that is attached to the end of the
+password.  The default is a newline.
+
 B<timeout> - Sets the timeout value for your scp operation. The default
 is 10 seconds.
 
@@ -416,7 +431,9 @@ many seconds *whether or not the operation has completed*.  Caveat programmor.
 
 B<user> - The login name you wish to use.
 
-B<verbose> - Set to 1 if you want verbose output sent to STDOUT.
+B<verbose> - Set to 1 if you want verbose output sent to STDOUT.  Note that
+this disables some error checking (ala no_check) because the verbose output
+could otherwise be picked up by expect itself.
 
 =head1 NOTES
 
@@ -431,35 +448,29 @@ a more secure way to perform scp.  Second, this module is not fast,
 even with error checking turned off.  Both reasons have to do with TTY
 interaction.
 
-Don't whine to me about putting passwords in scripts.  Set your
-permissions appropriately or use a .rc file of some kind.
-
 =head1 FUTURE PLANS
 
 There are a few options I haven't implemented.  If you *really* want to
 see them added, let me know and I'll see what I can do.
 
-A test suite (yes, I almost have one together) - no really, I promise!
-
 =head1 KNOWN BUGS
 
 At least one user has reported warnings related to POD parsing with Perl 5.00503.
 These can be safely ignored.  They do not appear in Perl 5.6 or later.
 
-I have one unconfirmed report of problems with wildcard characters.  I haven't
-had a chance to test this yet.
-
-There is a parsing error if you try to insert the host/ip in a 'from/to' string
-within an 'scp()' call.  The workaround is to specify the host with the '-host'
-option and just use ':' in the 'scp()' call string.  This will be fixed in the
-next release.
+See the README file for more on possible bugs you may encounter.
 
 =head1 THANKS
 
 Thanks to Roland Giersig (and Austin Schutz) for the Expect module.  Very handy.
 
+Thanks also go out to all those who have submitted bug reports and/or patches.
+See the Changes file for specifics.
+
 =head1 AUTHOR
 
 Daniel Berger
 
-djberg96@hotmail.com
+djberg96@yahoo.com
+
+rubyhacker1 on IRC
